; Laplace demo - declarations omitted for clarity.  File:  laplace.lisp

(in-package '*lisp)

;; Flag to distinguish interior from boundary

(*defvar interior
  (and!! (<!! (!! 0) (self-address-grid!! (!! 0)) (!! (1- (dimension-size 0))))
	 (<!! (!! 0) (self-address-grid!! (!! 1)) (!! (1- (dimension-size 1))))
   ))

;; Run n iterations of point-Jacobi iteration for the Laplace equation

(*defun laplace (gz n)
 (dotimes (k n)
  (*when interior
    (*set gz 
     (/!! 
      (+!! (news!! gz  0 -1) (news!! gz  1  0)
           (news!! gz  0  1) (news!! gz -1  0))
      (!! 4))))
 ) ; end dotimes
)

;; Initialize u with bilinear function on boundary, average of boundary
;; values in interior.

(*defvar u (!! 0.0))    ; Function value
(*defvar z (!! 0.0))    ; Solution

(defun init (a b c)
 (*let ((i (self-address-grid!! (!! 0)))
        (j (self-address-grid!! (!! 1))))
  (let ((w 0.0) (n 0))
    (*set z (+!! (*!! (!! a) i) (*!! (!! b) j) (!! c)))
    (*if (not!! interior)
       (progn (*set u z)
	      (setq w (*sum u)) (setq n (*sum (!! 1))))
       (*set u (!! (/ w n))) ; average of boundary values
    )) ; end *if, end let
 ) ; end *let
)

;; Maximum absolute difference between x, y

(*defun err (x y)
  (*max (abs!! (-!! x y)))
)

;; Run n iterations and examine errors

(defun try (n)
 (*let ((u0 u))
   (laplace u n)
   (list 'CHANGE (err u u0) 'ERROR (err u z))
 )
)

(setq *ppp-default-mode* :grid *ppp-default-format* "~6,2F ")
